#include <p24FJ128GB106.h>
#include "i2c.h"
#include "init.h"
#include "definitions.h"
#include "EEPROM.h"

unsigned char read_EEPROM(unsigned char address)
{
    unsigned char value=0;

    if(address > 0x7F){ return FALSE; }
    
    #ifdef I2C_HW
	
    I2C_Start();
    I2CsendByte(EEPROM_WRITE);
    I2CsendByte(address);
    I2C_Stop();
    I2C_Start();
    I2CsendByte(EEPROM_READ);

    value = I2C_getByte(FALSE);

    NotAckI2C1();
    I2C_Stop();
    return value;
	
    #else

    I2C_Start();
    I2CsendByte(EEPROM_WRITE);
    I2CsendByte(address);
    I2C_Stop();
    I2C_Start();
    I2CsendByte(EEPROM_READ);
    SDA_TRIS = 1;
    for(i=0 ; i<=7 ; i++){
        value |= (char)(SDA_READ <<(7-i));
        SCL = 0;
        SCL = 1;
        SCL = 0;
    }
    SDA_TRIS = 0;
    I2C_Stop();
    return value;

    #endif
}

int write_EEPROM(unsigned char address, unsigned char to_write)
{
    unsigned char write_check = 0;

    address &= EEPROM_SIZE; //con esto limitamos la direccion hasta 0x7F que es la locacion maxima

    I2C_Start();
    I2CsendByte(EEPROM_WRITE);
    I2CsendByte(address);
    I2CsendByte(to_write);
    I2C_Stop();

    write_check = read_EEPROM(address);

    if(write_check == to_write){
        return TRUE;
    }else{
        return FALSE;
    }
}

int test_EEPROM(void)
{
    //funcion que prueba la eeprom al escribirle valores random y despues regresandola a FF
    int errors=0;
    unsigned char index;
    unsigned char values[10] = {0xAA, 0x88, 0x35, 0x64, 0x84, 0xFE, 0xBB, 0x66, 0x21, 0xCD};

    for(index=0 ; index < 10 ; index++){
        while(!write_EEPROM((0x31+index), values[index]));

        if(read_EEPROM((0x31+index)) != values[index]){
            errors++;
        }
    }

    for(index=0 ; index < 128 ; index++){
        delayms(1);
        while(!write_EEPROM(index, 0xFF));
        
        if(read_EEPROM(index) != 0xFF){
            errors++;
        }
    }

    return errors;
}

