#include <p24FJ128GB106.h>
#include "i2c.h"
#include "init.h"
#include "definitions.h"
#include "LCD.h"


/*caracteres especiales que se guardan en la CGRAM del display
  hay 8 direcciones de memoria disponibles para guardar, con 
  la funcion load_custom_character, los siguientes arreglos de
  variables son caracteres modificados para desplegar el logo
  de Vinagron Digital.
*/

const unsigned char vina1[] = {	//tenaza izquierda
	0x04, 
	0x0C, 
	0x1C, 
	0x14, 
	0x1C, 
	0x0C,
	0x0E,
	0x0F,
};
	
const unsigned char vina2[] = {	//tenaza derecha
	0x00, 
	0x00,
	0x00, 
	0x08, 
	0x0C, 
	0x0C, 
	0x14, 
	0x18,
};
	
const unsigned char vina3[] = { //torax
	0x1F, 
	0x0F,
	0x1E, 
	0x0F,
	0x1C, 
	0x0E,
	0x18, 
	0x18,
};

const unsigned char di[] = {
	0x19, 
	0x15, 
	0x15, 
	0x15, 
	0x19, 
	0x00, 
	0x00, 
	0x00,
};

const unsigned char gi[] = {
	0x0D, 
	0x11, 
	0x15, 
	0x15, 
	0x19, 
	0x00, 
	0x00, 
	0x00,
};

const unsigned char t[] = {
	0x1F, 
	0x04, 
	0x04, 
	0x04, 
	0x04, 
	0x00, 
	0x00, 
	0x00,
};

const unsigned char al[] = {
	0x09, 
	0x15, 
	0x1D, 
	0x15, 
	0x15, 
	0x00, 
	0x00, 
	0x00,
};
//fin de los caracteres modificados

void LCDDelayms(unsigned int duration) // delay in -miliseconds
{
    unsigned int i;
    for(;duration!=0;duration--)
    {
        for(i=0;i<=114;i++)
        {
            asm("nop");
            asm("nop");
        }
        asm("nop");
        asm("nop");
    }
}

static char *i2a(unsigned i, char *a, unsigned r)
{
      if (i/r > 0) a = i2a(i/r,a,r);
      *a = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i%r];
      return a+1;
}

//char *itoa1(int i, char *a, int r)          //convierte integer a ASCII
char *itoa1(int i, unsigned char *a, int r)          //convierte integer a ASCII
{
      if ((r < 2) || (r > 36)) r = 10;
      if (i<0) {
            *a = '-';
            *i2a(-(unsigned)i,a+1,r) = 0;
      } else *i2a(i,a,r) = 0;

      return a;
}

void ClearDisplay(void)
{
    I2C_Start();
    I2CsendByte(0x78);              //ADDRESS
    I2CsendByte(0x00);              //comando
    I2CsendByte(0x01);              //LUGAR
    I2C_Stop();
}

void setCursor(unsigned char position)//pone el cursor en la pos. deseada
{ 
    I2C_Start();
    I2CsendByte(0x78);              //ADDRESS
    I2CsendByte(0x00);              //comando
    I2CsendByte(0x80 + position);   //LUGAR
    I2C_Stop();
}

void BlinkOn()
{ 
    I2C_Start();
    I2CsendByte(0x78);              //ADDRESS
    I2CsendByte(0x00);              //comando
    I2CsendByte(0x0F);
    I2C_Stop();
}

void BlinkOff()
{ 
    I2C_Start();
    I2CsendByte(0x78);              //ADDRESS
    I2CsendByte(0x00);              //comando
    I2CsendByte(0x0C);
    I2C_Stop();
}

void LCDwrite(char text)//send sttring of ASCII data to LCD
{
    I2C_Start();
    I2CsendByte(0x78);              //Slave, LCD address=0x78
    I2CsendByte(0x40);              //Datasend=0x40
    I2CsendByte(text);              //string
    I2C_Stop();
}

void goto_xy(int x, int y)
{
    switch(x){
        case 1: setCursor(0x00 + y);	break;
        case 2: setCursor(0x40 + y);	break;
    }
}

void DisplayString(unsigned char x, unsigned char y, unsigned char *m, unsigned char clear)
{		
    if(clear){
        ClearDisplay();
        LCDDelayms(1);
    }
    LCDDelayms(1);

    switch(x){
        case 1: setCursor(0x00 + y);	break;
        case 2: setCursor(0x40 + y);	break;
    }
    LCDDelayms(1);

    while(*m){
        LCDwrite(*m);
        *m++;
    }
}

void LCDIntro(){
    unsigned char i;       //01234567890123456789
    unsigned char let1[] = {"Aguij\xE2n"};  //el \xE2 le asigna la ""
    unsigned char let2[] = {"  3.0  "};
    unsigned char let3[] = {"inagr\xE2n"};


    load_custom_character(vina1, 0);	//|
    load_custom_character(vina2, 1);	//|----el vinagron
    load_custom_character(vina3, 2);	//|
    load_custom_character(di,3);		//|
    load_custom_character(gi,4);		//|----DIGITAL
    load_custom_character(t, 5);		//|
    load_custom_character(al,6);		//|

    LCDDelayms(1);

    goto_xy(1,0);
    LCDwrite(0);
    goto_xy(1,1);
    LCDwrite(1);
    goto_xy(2,0);
    LCDwrite(2);

    DisplayString(1,2,let3,FALSE);

    
    for(i=3 ; i<7 ; i++){       //escribir el logo de Vinagrn Digital
        goto_xy(2,(i+2));
        LCDwrite(i);
    }


    for(i=20 ; i!=12 ; i--){
        DisplayString(1,i,let1,FALSE);
        DisplayString(2,i,let2,FALSE);
        LCDDelayms(100);
    }

    LCDDelayms(2000);
    ClearDisplay();
}

void LCD_SendCommand(unsigned char command)	//enviar comando
{
    I2C_Start();
    I2CsendByte(0x78);//ADDRESS
    I2CsendByte(0x00);//
    I2CsendByte(command);
    I2C_Stop();
}

void load_custom_character (unsigned char *rows, unsigned char char_num) //cargar un caracter custom en CGRAM
{
    int i;

    if(char_num>=8) return; //we only have 8 spaces at CGRAM

    LCDDelayms(10);

    // Set up the display to write into CGRAM - configure LCD to use func table 0
    I2C_Start();
    I2CsendByte(0x78); //address
    I2CsendByte(0x00); //command
    I2CsendByte(0x38); //set to func table 0
    LCDDelayms(10);
    // Set CGRAM position to write
    I2CsendByte( 0x40 + (char_num*8));
    I2C_Stop();


    I2C_Start();
    I2CsendByte(0x78);
    I2CsendByte(0x40);    //RS=1 (write data to ram)

    //send the 8 rows of the character
    for(i=0;i<8;i++)
    {
        I2CsendByte(*rows); // write the current pointed row to CGRAM
        rows++;             //increment the pointer
    }

    I2C_Stop();
    LCDDelayms(10);

    // Leave the LCD as it was - function table 1 DDRAM and set the cursor
    LCD_SendCommand (0x39);

    LCDDelayms(10);
}
