#include <p24FJ128GB106.h>
#include "definitions.h"
#include "analog.h"

#define ADC_MAX_VALUE   1017
#define ADC_MIN_VALUE   0


void initADC(void)
{
    AD1PCFG = 0x7FFF; 		// select RB15(AN15) as analog input
    AD1CON1 = 0x00E0; 		// internal counter ends sampling and starts conversion
    AD1CON2 = 0;		// use AVDD, AVSS as reference pins
    AD1CON3 = 0x1F02;		// Tsamp = 32 x Tad; Tad=125ns
    AD1CSSL = 0; 		// no scanning required
    AD1CON1bits.ADON = 1; 	// turn on the ADC
} //initADC

int readADC(void)
{
    AD1CHS = 15;		// 1. select analog input channel
    AD1CON1bits.SAMP = 1; 	// 2. start sampling
    while (!AD1CON1bits.DONE); 	// 3. wait for the conversion to complete
    return ADC1BUF0; 		// 4. read the conversion result
} // readADC

int normalize_ADCReading(unsigned char inferiorLimit, unsigned char superiorLimit)
{
    /*normaliza el valor del ADC con las siguientes ecuaciones:
     *   x = MAX_VALUE / (superiorLimit - inferiorLimit)
     *
     *   (LecturaADC / x) + 10
     *
     */
    float x = 0.0;
    float preResult = 0.0;
    int ADCreading  = 0;
    int finalResult = 0;

    ADCreading = readPOT();

    x = ADC_MAX_VALUE / (superiorLimit - inferiorLimit);

    preResult  = (ADCreading / x);
    preResult += 10;

    finalResult = (int)(preResult);

    return finalResult;
}
