#include <p24FJ128GB106.h>
#include "GenericTypeDefs.h"    //for BOOL data type
#include "definitions.h"
#include "inputs.h"
#include "outputs.h"
#include "LCD.h"
#include "init.h"
#include "analog.h"
#include "EEPROM.h"
#include "demo.h"

BOOL toggle  = TRUE;
BOOL toggle2 = TRUE;
unsigned char backlight_index=4;


unsigned char message[] = " ";

extern unsigned char minutes;
extern unsigned char seconds;

typedef enum __MAIN_STATES{
    TIME,
    INPUT,
    OPTO,
    EEPROM,
} MAIN_STATES;

static MAIN_STATES demo_state = 0;


/******** Private function protorypes ************/
void TimeDemo(void);
void InputDemo(void);
void OptoDemo(void);
void EEPROM_demo(void);
/********* Ends prototypes ***********************/

static unsigned char keyboard = 0;


void keyPressHandler(void)
{
    char i;

    keyboard = readSW();

    //prender los cuatro relays cuando el SW1 sea presionado.
    if(keyboard == 1 && demo_state != EEPROM){
        if(toggle){
            for(i=0 ; i<4 ; i++){
                delayms(150);
                RLYport(i+1,ON);
            }
            toggle = FALSE;
        }else{
            for(i=0 ; i<4 ; i++){
                delayms(150);
                RLYport(i+1,OFF);
            }
            toggle = TRUE;
        }
    }

    //cada vez que el SW2 sea presionado, cambiar el brillo del backlight
    if(keyboard == 2){
        delayms(200);
        buzzer(400,50);
        switch(backlight_index){
            case 1: backlight(BLI1); backlight_index++; break;
            case 2: backlight(BLI2); backlight_index++; break;
            case 3: backlight(BLI3); backlight_index++; break;
            case 4: backlight(BLI4); backlight_index++; break;
            case 5: backlight(0);    backlight_index=1; break;
        }
        delayms(200);
        Buzzer_OFF();
    }

    //prender todo el OCport con una secuencia LSB -> MSB
    if(keyboard == 3){
        buzzer(450,50);
        delayms(300);
        Buzzer_OFF();
        if(toggle2){
            for(i=0 ; i<4 ; i++){
                delayms(150);
                OCport(i+1,ON);
            }
            toggle2 = FALSE;
        }else{
            for(i=0 ; i<4 ; i++){
                delayms(150);
                OCport(4-i,OFF);
            }
            toggle2 = TRUE;
        }
    }

    //cambiar de estado y, hacer un beep
    if(keyboard == 4){
        buzzer(500,50);
        delayms(400);
        Buzzer_OFF();

        if(demo_state < STATE_COUNT){ demo_state++;  }
        else                        { demo_state = 0;}

    }
}


unsigned char get_state(void)
{
    return demo_state;
}

void set_state(unsigned char value)
{
    if(value <= STATE_COUNT){
        demo_state = value;
    }else{
        demo_state = 1;
    }
}

void DemoMenu(void) //main state machine for main program demo
{       
    switch(demo_state){
        case TIME:    TimeDemo();    break;
        case INPUT:   InputDemo();   break;
        case OPTO:    OptoDemo();    break;
        case EEPROM:  EEPROM_demo(); break;
    }
}

void TimeDemo(void)
{
    unsigned char msg1[] = {"       00:00        "};
    unsigned char msg2[] = {"   Realtime Clock   "};

    msg1[7] = (minutes / 10) | 0x30;    //obtener minutos y convertirlos a ASCII
    msg1[8] = (minutes % 10) | 0x30;

    msg1[10] = (seconds / 10) | 0x30;   //igual con los segundos
    msg1[11] = (seconds % 10) | 0x30;

    DisplayString(1,0,msg1,FALSE);
    DisplayString(2,0,msg2,FALSE);
}

void InputDemo(void)    //escribe en la LCD al valor ADC y del DIPSW
{
    unsigned char msg1[] = {"       ->DIP SW     "};
    unsigned char msg2[] = {"       ->ADC        "};
    unsigned char value = 0;
    unsigned int adcval = 0;

    value  = readDIP();      //leer el DIP switch
    adcval = readPOT();     //leer la entrada AN1



    DisplayString(1,0,msg1,FALSE);
    DisplayString(2,0,msg2,FALSE);

    LATE = value;                //asignar el valor de los leds al del dip
    LATE ^= 0xFF;

    if(value > 0){
        itoa1(value,message,16); //convertir el valor a ASCII y en hexadecimal
        DisplayString(1,0,message,FALSE);
    }

    if(adcval > 0){
        itoa1(adcval,message,16);
        DisplayString(2,0,message,FALSE);
    }
}

void OptoDemo(void) //escribe en la LCD cual entrada se activo
{
    unsigned char msg1[] = {"  Opto     IN1  IN2  "};
    unsigned char msg2[] = {"Acopladas  IN3  IN4  "};

    if(PORTE != 0){
        LATE = 0x00FF;  //apagar todos los leds
    }

    if(!OPTO1){
        msg1[10] = '['; //ponerle [ ] para sealar cual se activo
        msg1[14] = ']';
    }
    if(!OPTO2){
        msg1[15] = '[';
        msg1[19] = ']';
    }
    if(!OPTO3){
        msg2[10] = '[';
        msg2[14] = ']';
    }
    if(!OPTO4){
        msg2[15] = '[';
        msg2[19] = ']';
    }

    DisplayString(1,0,msg1,FALSE);
    DisplayString(2,0,msg2,FALSE);
}

void EEPROM_demo(void)
{
    int i2c = 0;
    BOOL test_done = FALSE;
    unsigned char temp_tecla;

    unsigned char msg1[]    = {"Test EEPROM -> SW1  "};
    unsigned char msg2[]    = {"Press SW1           "};

    unsigned char msgTEST[] = {"Testing EEPROM...   "};
    unsigned char msgOK[]   = {"No   R/W errors     "};
    unsigned char msgBAD[]  = {"     R/W errors     "};

    DisplayString(1,0,msg1,FALSE);

    if(read_EEPROM(0x22) == 0xCC){
        DisplayString(2,0,msgOK,FALSE);
    }else if(!test_done){
        DisplayString(2,0,msg2,FALSE);
    }

    if(!SW1){
        DisplayString(2,0,msgTEST,FALSE);
        delayms(1000);

        i2c = test_EEPROM();

        if(i2c > 0){
            itoa1(i2c, message, 10);
            DisplayString(2,0,msgBAD,FALSE);
            DisplayString(2,0,message,FALSE);
            while(!write_EEPROM(0x22, 0xAA));
        }else{
            DisplayString(2,0,msgOK,FALSE);
            while(!write_EEPROM(0x22, 0xCC));
        }
        test_done = TRUE;
    }
}
