#include <p24FJ128GB106.h>
#include "i2c.h"
#include "definitions.h"
#include "init.h"


void configTimer1(void)
{
    T1CON = 0x00;               //Stops the Timer1 and reset control reg.
    TMR1  = 0x00; 		//Clear contents of the timer register
    PR1   = 0x1F6F; 		//Load the Period register with 1mSec preset
    IPC0bits.T1IP = 0x01; 	//Setup Timer2 interrupt for desired priority level

    IFS0bits.T1IF = 0; 		//Clear the Timer1 interrupt status flag
    IEC0bits.T1IE = 1; 		//Enable Timer1 interrupts
    T1CONbits.TON = 1; 		//Start Timer1 with prescaler settings at 1:1 and
    //clock source set to the internal instruction cycle
}

void InitI2C(void)
{
    unsigned int config2, config1;

    SCL_TRIS = 0;
    SDA_TRIS = 0;
    RST_TRIS = 0;


    /* Baud rate is set for 100 Khz */
    config2 = 0x27;
    /* Configure I2C for 7 bit address mode */
    config1 = (I2C_ON & I2C_IDLE_CON & I2C_CLK_HLD & I2C_IPMI_DIS & I2C_7BIT_ADD
             & I2C_SLW_DIS & I2C_SM_DIS & I2C_GCALL_DIS & I2C_STR_DIS & I2C_NACK
             & I2C_ACK_DIS & I2C_RCV_DIS & I2C_STOP_DIS & I2C_RESTART_DIS & I2C_START_DIS);

    OpenI2C1(config1,config2);
    IdleI2C1();
}

#ifdef I2C_HW	//estamos usando I2C por hardware?

//I2C por hardware

void I2C_Start(void)
{
    StartI2C1();             \
    while(I2C1CONbits.SEN); /* Wait till start sequence is completed */
}

void I2C_Stop(void)
{
    StopI2C1();
    while(I2C1CONbits.PEN); /* Wait till stop sequence is completed */
}

void I2CsendByte(unsigned char j)
{
    MasterWriteI2C1(j);         // Write Slave address and set master for transmission
    /* Wait till address is transmitted */
    while(I2C1STATbits.TBF);    // 8 clock cycles
    while(!IFS1bits.MI2C1IF);   // Wait for 9th clock cycle
    IFS1bits.MI2C1IF = 0;       // Clear interrupt flag
}

unsigned char I2C_getByte(char do_ack)
{
    unsigned char data=0;

    I2C1CONbits.RCEN = 1;
    while(I2C1CONbits.RCEN);
    I2C1CONbits.RCEN   = 0;
    I2C1STATbits.I2COV = 0;
    data = I2C1RCV;

    if(do_ack){
        AckI2C1();
        while(I2C1CONbits.ACKEN);
    }
    return data;
}

#else   //si no, hacerlo por software

//I2C por software

void I2C_Stop(void)
{
    SDA=0;
    SCL=0;
    SCL=1;
    SDA=1;
}

void I2C_Start(void)
{
    SCL=1;
    SDA=1;
    SDA=0;
    SCL=0;
}

void I2CsendByte(unsigned char j) //I2C Output
{
    int n;
    volatile char SDA_VAR;// = 1;
    unsigned char d;
    int retries=200;


    d=j;
    for(n=0;n<8;n++){
        if((d&0x80)==0x80)
                SDA=1;
        else
                SDA=0;
        d=(d<<1);
        SCL = 0;
        SCL = 1;
        SCL = 0;
    }

    SDA_TRIS = 1;

    SDA_VAR = SDA_READ;
    SCL = 1;
    while(SDA_VAR==1 && retries){
        SDA_VAR = SDA_READ;
        SCL=0;
        SCL=1;
        retries--;
    }

    SDA_TRIS = 0;
    SCL=0;
}
#endif

void LCDinit(void)			//initialization for ST7036i
{
    RST = 1;

    LCDDelayms(80);

    I2C_Start();
    I2CsendByte(0x78);//Slave=0x78
    I2CsendByte(0x00);//Comsend = 0x00
    I2CsendByte(0x38);
    LCDDelayms(10);
    I2CsendByte(0x39);
    LCDDelayms(10);
    I2CsendByte(0x14);
    I2CsendByte(0x70);
    I2CsendByte(0x5E);
    I2CsendByte(0x6D);
    I2CsendByte(0x0C);
    I2CsendByte(0x01);
    I2CsendByte(0x06);
    LCDDelayms(10);
    I2C_Stop();
}

void LCDinit2(void)			//letras grandes
{
    RST = 1;

    LCDDelayms(10);

    I2C_Start();
    I2CsendByte(0x78);//Slave=0x78
    I2CsendByte(0x00);//Comsend = 0x00
    I2CsendByte(0x34);
    LCDDelayms(10);
    I2CsendByte(0x34);
    LCDDelayms(10);
    I2CsendByte(0x14);
    I2CsendByte(0x70);
    I2CsendByte(0x5E);
    I2CsendByte(0x6D);

    LCDDelayms(10);
    I2C_Stop();
}
