#include <p24FJ128GB106.h>
#include <GenericTypeDefs.h>
#include "definitions.h"
#include "inputs.h"



void DIPconfig(unsigned char cfg)
{
    switch(cfg){
        case DMSB:
            DSWMSBE = 0;		//habilitar MSB
            DSWLSBE = 1;		//deshabilitar LSB
            break;
        case DLSB:
            DSWMSBE = 1;		//deshabilitar MSB
            DSWLSBE = 0;		//habilitar LSB
            break;
        case DDIS:
            DSWMSBE = 1;                //<-|-DIP SW disable
            DSWLSBE = 1;                //<-+
            break; 
    }
}

unsigned char readDIP(void)	//se encarga de regresar el valor del dip switch
{
    unsigned char i, result=0;

    for(i=0 ; i<8 ; i++){
        switch(i){
            case 0:				//MSB
                DIPconfig(DMSB);                //habilita los MSB del DIP
                if(!DIP1_5){result += 128;}
                break;
            case 1:
                if(!DIP2_6){result += 64;}
                break;
            case 2:
                if(!DIP3_7){result += 32;}
                break;
            case 3:
                if(!DIP4_8){result += 16;}
                break;
            case 4:				//LSB
                DIPconfig(DLSB);                //enable LSB
                if(!DIP1_5){result += 8; }
                break;
            case 5:
                if(!DIP2_6){result += 4; }
                break;
            case 6:
                if(!DIP3_7){result += 2; }
                break;
            case 7:
                if(!DIP4_8){result += 1; }
                break;
        }
    }

    DIPconfig(DDIS);		//Disable DIP SW

    return result;
}

BOOL DebounceKeyb(void)
{
    if(!SW1 || !SW2 || !SW3 || !SW4){               //any SWx pressed
        delayms(10);                                //wait 10 mS
        if(!SW1 || !SW2 || !SW3 || !SW4){           //is it still pressed?
            return TRUE;                            //yes, we have a valid key press
        }else{
            return FALSE;                           //no valid keypress
        }
    }
}

BOOL DebounceOpto(void)
{
    if(!OPTO1 || !OPTO2 || !OPTO3 || !OPTO4){       //any OPTOx Active
        delayms(10);                                //wait 10mS
        if(!OPTO1 || !OPTO2 || !OPTO3 || !OPTO4){   //still active?
            return TRUE;                            //yes, valid input
        }else{
            return FALSE;                           //not a valid input
        }
    }
}

unsigned char getActiveSW(void)
{
    unsigned char tecla = 0;

    if(!SW1)     { tecla = 1; } //se guarda en "tecla" el valor del switch presionado
    else if(!SW2){ tecla = 2; } //se presiono el 2
    else if(!SW3){ tecla = 3; } // ... 3
    else if(!SW4){ tecla = 4; } // ... el 4
    else{
        return 0;               //no hay switches presionados
    }
    
    if(!DebounceKeyb()){        //al parecer todo va bien, hacer debounce
        return 0;               //hubo un error en la rutina de debounce
    }    
    return tecla;               //todo sale bien, regresar el valor del switch
}

unsigned char getActiveOI(void)
{
    unsigned char input = 0;

    if(!OPTO1)     { input = 1; } //se guarda en "tecla" el valor del switch presionado
    else if(!OPTO2){ input = 2; } //se presiono el 2
    else if(!OPTO3){ input = 3; } // ... 3
    else if(!OPTO4){ input = 4; } // ... el 4
    else{
        return 0;               //no hay switches presionados
    }

    if(!DebounceOpto()){        //al parecer todo va bien, hacer debounce
        return 0;               //hubo un error en la rutina de debounce
    }
    return input;               //todo sale bien, regresar el valor del switch
}

unsigned char readSW(void)
{
    static unsigned char pressed     = 0;   //guarda el valor de la tecla actualmente presionada
    static unsigned char lastPressed = 0;   //guarda el valor de la tecla que fue presionada

    //se activan en 0 !SWx
    pressed = getActiveSW();   //saber cual sw esta presionado y hacer debounce

    if(pressed){
        if(pressed == lastPressed){
            return 0;
        }else{
            lastPressed = pressed;
            return pressed;
        }
    }else{
        if(lastPressed != 0){
            lastPressed = 0;
        }
        return 0;
    }
    return 0;   
}

unsigned char readOI(void)
{
    static unsigned char active     = 0;   //guarda el valor de la entrada activa
    static unsigned char lastActive = 0;   //guarda el valor de la entrada que estaba activa

    //se activan en 0 !OPTOx
    active = getActiveOI();   //saber cual oi esta activada y hacer debounce

    if(active){
        if(active == lastActive){
            return 0;
        }else{
            lastActive = active;
            return active;
        }
    }else{
        if(lastActive != 0){
            lastActive = 0;
        }
        return 0;
    }
    return 0;
}
