/*

	Aguijn rev 3.0(x128kb) by JCOG
	 Vinagrn Digital 2012
	
	**********************************************
	Para informacion adicional y/o documentacion,
	ver el CD de documentacion o entrar a la 
	siguiente direccion:
	
            
 *         http://www.vinagrondigital.com

	
	**********************************************
*/ 


#include <p24FJ128GB106.h>  //PIC24 header
#include "EEPROM.h"         //EEPROM library
#include "analog.h"         //analog library
#include "LCD.h"            //LCD library
#include "outputs.h"        //output library
#include "inputs.h"         //input library
#include "serial.h"         //RS232 Library
#include "PPS.h"            //remappable pins
#include "realtime.h"       //RTCC
#include "outcompare.h"     //PWM
#include "init.h"           //init library
#include "tester.h"         //RS232 Tester
#include "demo.h"           //Demo Code
#include "definitions.h"    //Global definitions



BOOL TESTMODE        = FALSE;
BOOL refresh_display = FALSE;

extern unsigned char cmdBuffer[];
unsigned char string[20] = " ";

unsigned char minutes  = 0;
unsigned char seconds  = 0;
unsigned char sweeppos = 0;


unsigned char testmsg1[] = {"   test mode 1.0    "};
unsigned char testmsg2[] = {"Waiting for COMMands"};



_CONFIG1(JTAGEN_OFF & FWDTEN_OFF & ICS_PGx2)
_CONFIG2(FNOSC_PRIPLL & POSCMOD_HS& PLL_96MHZ_ON & PLLDIV_DIV2 & FCKSM_CSECMD)

void delayms(unsigned int duration) // delay in -miliseconds
{
    unsigned int i;
    for(;duration!=0;duration--)
    {
        for(i=0;i<=114;i++)
        {
            asm("nop");
            asm("nop");
        }
        asm("nop");
        asm("nop");
    }
}

void Aguijon3Init(void)
{
    //initialize ext crystal
    CLKDIVbits.DOZEN = 1;
    CLKDIVbits.DOZE0 = 0;
    CLKDIVbits.DOZE1 = 1;
    CLKDIVbits.DOZE2 = 0;

    CLKDIVbits.CPDIV0 = 1;
    CLKDIVbits.CPDIV1 = 0;
    //end of OSC config

    //initialize I/O ports
    TRISB = 0xBDCF;		//configure PORTB
    LATB  = 0x0000;

    TRISD = 0x0000;		//configure PORTD as outputs
    LATD  = 0x0000;

    TRISE = 0x0000;		//configure PORTE as outputs
    LATE  = 0x0000;

    TRISG = 0x0000;		//configure PORTG as outputs
    LATG  = 0x0000;

    TRISF = 0x0000;		//configure PORTF as outputs
    LATF  = 0x0000;
    //done

    //initialize ADC(CH15 POT) & display
    initADC();		

    #ifdef I2C_HW
        InitI2C();
    #endif
    LCDinit();
    //done

    //disable DIP Switch
    DIPconfig(DDIS);


    PPSUnLock;
    //initialize pins for RS232 Rx & Tx
    iPPSInput(IN_FN_PPS_U2RX,IN_PIN_PPS_RP8);       //Assing U2RX to pin RP8
    iPPSOutput(OUT_PIN_PPS_RP9,OUT_FN_PPS_U2TX);    //Assing U2TX to pin RP9
    initUARTcommunication();                        //init RS232C driver
    //done

    //initialize PWM for buzzer & display
    PPSOutput(PPS_RP21, PPS_OC1);                   //RP21 selected as OC1 output pin for backlight
    PPSOutput(PPS_RP26, PPS_OC2);                   //RP26 selectes as OC2 output pin for buzzer
    CloseOC1();
    CloseOC2();
    ConfigIntOC1(OC_INT_OFF);
    ConfigIntOC2(OC_INT_OFF);
    PPSLock;
    //done?

    backlight(BLI3);                                //prender el backlight en la intensidad mas baja
    LCDIntro();                                     //intro con el LCD

//    LCDinit2();                                     //letras grandes en el display
    LATE = 0x00FF;                                  //apagar el puerto de leds

    configTimer1();                                 //prender el timer 1
    initRTCC();                                     //prender el RTCC

}

//Interrupcin se llama cada milisegundo
void __attribute__((__interrupt__, __shadow__)) _T1Interrupt(void)
{	
    static unsigned int t1subsec = 0;

    t1subsec++;

    if(t1subsec == 99){                     //cada 100 ms
        refresh_display = TRUE;
        t1subsec = 0;

        if(get_state() == 0){
            //secuencia de los leds
            if(sweeppos < 7){ sweeppos++; }
            else{ sweeppos = 0; }

            LATE = (0x0080>>sweeppos);
            LATE ^= 0xFF;
        }
        
    }

    IFS0bits.T1IF = 0; //Reset Timer1 interrupt flag and Return from ISR
}

void __attribute__ ((interrupt,no_auto_psv)) _RTCCInterrupt (void)
{
    if(seconds<59){
        seconds++;
    }else{ //the 60th second
        seconds=0;
        if(minutes<99){ //the 100th second
            minutes++;
        }else{
            minutes = 0;
        }
    }

    mRtcc_Clear_Intr_Status_Bit;//clear the interrupt status
}

int main(void)
{		
    Aguijon3Init();             //inicializa el board
        
    if(readDIP() == 0x81){              //entrar al modo prueba del board
        TESTMODE = TRUE;
        T1CON = 0x00; 			//Stops the Timer1 and reset control reg.
	TMR1  = 0x00; 	 		//Clear contents of the timer register+
        DisplayString(1,0, testmsg1, FALSE);
        DisplayString(2,0, testmsg2, FALSE);
        WriteUART2(0xFF);               //un FF para indicar que estamos listos
    }

    while(TESTMODE){					//test mode 1.0
        /* recibe comandos y los interpreta
         * para probar los modulos de la
         * tablilla. los comandos son recibidos
         * por RS232.
         */
        switch(cmdBuffer[0]){
            case RELAY:  RLY_serial();      break;
            case OCOUT:  OC_serial();       break;
            case LEDPT:  LEDPort_serial();  break;
            case BUZZ:   /*unimplemented*/  break;
            case OII:    /*unimplemented*/  break;
            case DIPSW:  /*unimplemented*/  break;
            case PT10K:  /*unimplemented*/  break;
        }
    }

    for(;;)
    {       
        //escribir la LCD cada vez que refresh_display sea TRUE
        if(refresh_display){            
            DemoMenu();
            refresh_display = FALSE;    //el timer se encarga de lo demas
        }

        keyPressHandler();
    }
}
