#include <p24FJ128GB106.h>
#include "realtime.h"
#include "definitions.h"


/*****************************************************************************
 * Function: RTCCUnlock
 *
 * Preconditions: None.
 *
 * Overview: The function allows a writing into the clock registers.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void RTCCUnlock(void){
    __builtin_write_RTCWEN(1); //set the RTCWREN bit
    asm volatile("bset	_RCFGCAL, #13");
    __builtin_write_RTCWEN(0); //set the RTCWREN bit
}

/*****************************************************************************
 * Function: RTCCSet
 *
 * Preconditions: None.
 *
 * Overview: This function sets the time, date, day of week, and year
 *
 * Input: 
 *
 * Output: None.
 *
 *****************************************************************************/
void RTCCSet(void)
{
    IEC3bits.RTCIE  = 1;        //enable RTCC interrupt
    IPC15bits.RTCIP = 4;        //priority #5
    //***********************************************************
    //  STEP 1:
    //  Unlock RTCC Registers
    //***********************************************************/
    mRTCCUnlock();           // ###	use predefined unlock macro in realtime.h

    //***********************************************************
    //  STEP 2:
    //  Configure RTCPTR Autodecrementing pointer in RCFGCAL register
    //  to point the "-- : Year" address
    //***********************************************************/
    RCFGCALbits.RTCPTR = 3;	// ### setup RTCPTR pointer to max value

    //***********************************************************
    //  STEP 3:
    //  Write Year, Month & Day, Weekday & Hours,
    //  Minutes & Seconds To RTCVAL in BCD format
    //***********************************************************/
    RTCVAL = 0x0000;         // ### unused  : Year
    RTCVAL = 0x0000;         // ### Month   : Day
    RTCVAL = 0x0000;         // ### Weekday : Hours
    RTCVAL = 0x0000;         // ### Minutes : Seconds

    //***********************************************************
    //  STEP 4:
    //  In RCFGCAL, enable RTCC
    //***********************************************************/
    RCFGCALbits.RTCEN = 1;   // ### enable RTCC

    //***********************************************************
    //  STEP 5:
    //  Lock RTCC Registers
    //***********************************************************/
    mRTCCLock();             // ### use predefined lock macro in rtcc.h
}

/*****************************************************************************
 * Function: AlarmSet
 *
 * Preconditions: None.
 *
 * Overview: The function sets the alarm
 *
 * Input:
 *
 * Output: None.
 *
 *****************************************************************************/
void AlarmSet(void)
{
    //  UnLock Alarm Registers
    mRTCCUnlock();

    //  Configure ALCFGRPT, ALRMPTR Auto-decrementing pointer
    ALCFGRPTbits.ALRMPTR = 0;
    ALCFGRPTbits.CHIME   = 1;

    //  Write Alarm Month & Day To ALRMVAL
    ALRMVAL = 0x0000;

    //  Write Alarm Week Day & Hour To ALRMVAL
    ALRMVAL = 0x0000;

    //  Write Alarm Minute & Second To ALRMVAL
    ALRMVAL = 0x0000;

    //***********************************************************
    //  STEP 6:
    //  In ALCFGRPT, configure alarm frequency @ 1Hz (every 1s)
    //  In ALCFGRPT, configure alarm to repeat x times
    //***********************************************************/
    ALCFGRPTbits.AMASK = 1;     // ### setup for an alarm every 1s
    ALCFGRPTbits.ARPT  = 0xFF;  // ### repeat alarm n times
    //***********************************************************
    //  STEP 7:
    //  In ALCFGRPT, enable Alarm
    //***********************************************************/
    ALCFGRPTbits.ALRMEN = 1;    // ### enable alarm


    //  Lock Alarm Registers
    mRTCCLock();                // Lock the RTCC
}
/*****************************************************************************
 * Function: RTCCInit
 *
 * Preconditions: RTCCInit must be called before.
 *
 * Overview: Enable the oscillator for the RTCC
 *
 * Input: None.
 *
 * Output: None.
 *****************************************************************************/
void initRTCC(void)
{
    // Enables the LP OSC for RTCC operation
    __builtin_write_OSCCONL(0x02); //Ejecuta la secuencia de unlock y escibe un 0x02 al registro OSCCON
    
    // Unlock sequence must take place for RTCEN to be written
    RCFGCAL	= 0x0000;

    AlarmSet();

    mRTCCSet();
}
